<?php

namespace chatbro;

use \chatbro\traits\StoreLoader;

use \chatbro\chatbroapp\shortcodes\CbroShortCode;
use \chatbro\chatbroapp\backends\CbroUserBackend;
use \chatbro\chatbroapp\common\CbroCommon;
use \chatbro\chatbroapp\users\CbroUser;
use \chatbro\chatbroapp\admin\CbroAdmin;
use \chatbro\chatbroapp\utils\CbroUtils;
use \chatbro\chatbroapp\settings\CbroSettings;
use \chatbro\chatbroapp\chats\CbroChats;
use \chatbro\chatbroapp\api\CbroApi;
use \chatbro\chatbroapp\chats\CbroDisplayChat;
use \chatbro\chatbroapp\permissions\CbroPermissions;

use \chatbro\chatbroapp\backends\CbroUtilsBackend;
use \chatbro\chatbroapp\backends\CbroAdminBackend;
use \chatbro\chatbroapp\backends\CbroSettingsBackend;
use \chatbro\chatbroapp\backends\CbroChatsBackend;
use \chatbro\chatbroapp\backends\CbroApiBackend;
use \chatbro\chatbroapp\backends\CbroPermissionsBackend;

class Chatbrochats extends Library
{
    use StoreLoader;

    const DEBUG_LOG_FILENAME = 'chatbro.%s.log';

    public $store_url;
    public $store_name;
    public $store_id = 0;
    private $debug_log;

    public function __construct($registry, $store_id)
    {
        parent::__construct($registry);

        $this->store_id = $store_id;

        $this->load->model('setting/setting');

        if ($this->store_id === 0) {
            $this->store_url = basename(DIR_TEMPLATE) == 'template' ? HTTPS_CATALOG : HTTPS_SERVER;
            $this->store_name = $this->config->get('config_name');
        } else {
            $this->store_url = $this->model_setting_setting->getSettingValue('config_ssl', $store_id);
            $this->store_name = $this->model_setting_setting->getSettingValue('config_name', $store_id);
        }

        $this->loadStore($this->store_id);

        $this->debug_log = new Log(sprintf(self::DEBUG_LOG_FILENAME, $this->store_id));
    }

    public function getStoreUrl()
    {
        return $this->store_url;
    }

    public function admin()
    {
        return CbroAdmin::display();
    }

    public function getChats()
    {
        return CbroAdmin::get_chats();
    }

    public function deleteChat()
    {
        return CbroAdmin::delete_chat();
    }

    public function updateChat()
    {
        return CbroAdmin::update_chat();
    }

    public function createChat()
    {
        return CbroAdmin::create_chat();
    }

    public function saveSettings()
    {
        return CbroAdmin::save_settings();
    }

    public function getShortcodeInstance()
    {
        return CBroShortCode::get_instance();
    }

    public function displayChats()
    {
        $result = "";

        foreach (CbroChats::iterator() as $name => $chat) {
            $result .= (new CbroDisplayChat($chat))->get_sitewide_popup_chat_code();
        }

        return $result;
    }

    public function init()
    {
        CbroUtils::init(new CbroUtilsBackend($this));
        CbroCommon::init($this);
        CbroAdmin::init(new CbroAdminBackend());
        CbroSettings::init(new CbroSettingsBackend());
        CbroChats::init(new CbroChatsBackend());
        CbroApi::init(new CbroApiBackend());
        CbroUser::init(new CbroUserBackend($this));
        CbroPermissions::init(new CbroPermissionsBackend());

        if (!class_exists("CBroShortCode")) {
            CbroShortCode::get_instance();
        }

        CbroChats::init_chats();
    }

    public function debugLog($text)
    {
        $this->debug_log->write($text);
    }
}